
format short;

%  Correlational Analyses of Exchangeable-Case Dyadic Data
%  Griffin & Gonzalez, 1995, Psychological Bulletin, 118, 430-439.

%  the following command causes SPSS to read a data file where
%  the first collumn contains the dyad numbers (the dyad id# given
%  to both dyad members); the second collumn contains the person
%  id #s (e.g., sex) coded as 1 or 2; the third and subsequent
%  collumns contains the variables to be analyzed

%  the data files must have been sorted by dyad id# and by person id # 

%  there should be no missing values for any of the variables


%  data from Griffin & Gonzalez (1995)
data = [
  4   52.5     20      5
  4   43.5     25      3
  6   48.5     64     17
  6   41.0     44      4
  7   33.5     50      2
  7   34.5     19      2
 12   45.5     38      8
 12   64.5     21     10
 14   25.0     26      0
 14   26.0     20      2
 17   10.5      4      1
 17   11.5      6      1
 19   51.0     35     12
 19   51.5     22      6
 21   55.0     27     20
 21   63.0     21      3
 22   36.5     10      3
 22   38.5     19      5
 23   59.0     27      4
 23   38.5     37      2
 27   48.0     29      0
 27   53.5     11     22
 29   45.5     25      6
 29   55.0     25     13
 30   40.0      6      3
 30   36.0      4      0
 31   27.0     76      4
 31   32.5     37     10
 33   44.0     29      5
 33   34.0     14      2
 34   45.5     64     45
 34   67.5     33     13
 35   30.0      9      0
 35   29.5     14      0
 37    0.0      0      0
 37    0.0      1      1
 40   31.0     13      0
 40   50.5     21      3
 42   41.5     26      2
 42   42.0     27     11
 43   33.5     20      9
 43   29.0     11      3
 45   62.0     25     22
 45   57.0     21      9
 47   51.0     23      1
 47   58.0     22      5
 48    1.5      2      0
 48    1.5      5      0  ];

% Done.  The program will can now analyze the data.

norig = size(data,1);
tailed = 2;

% selecting pairs that have the same dyad #s
datarev=repmat(-9999,1,size(data,2));
for aa = 1:size(data,1)-1;
if data(aa,1)==data(aa+1,1);
datarev = [ datarev ; data(aa:(aa+1) ,:) ];
end;
end;
data = datarev(2:size(datarev,1),2:size(data,2));

n = size(data,1) / 2;

% setting up pairwise ("double-counted") collumns of data
xp = repmat(-9999,size(data,1),size(data,2));
for aa = 1:size(data,1)/2;
xp(aa*2-1,:) = data(aa*2  ,:);
xp(aa*2  ,:) = data(aa*2-1,:);
end;
m = [ data , xp ];

% correlation matrix
nr = size(m,1);
rawsp = m' * m ;
rsums = sum(m,1)';
corsp = rawsp - (1/nr) * (rsums) * rsums' ;
vcv = corsp * (1/(nr-1));
d = inv( diag( sqrt(diag(vcv)) ) );
cr = d * vcv * d;

overall = cr(1:(size(cr,1)/2) , 1:(size(cr,2)/2));
iccross = cr(((size(cr,1)/2)+1):size(cr,1), 1:(size(cr,1)/2));

% significance levels of the intraclass correlations
intra   = diag(iccross);
zintra  = intra * sqrt(n);
pzintra = (1 - normp(abs(zintra))) * 2;

% initializing
nstarovr   = repmat(-9999,size(overall,1),size(overall,2));
zover      = nstarovr;
pzover     = zover;
nstarcrs   = zover;
zcrs       = zover;
pzcrs      = zover;
indcorrs   = zover;
tind       = zover;
ptind      = zover;
dyadcors   = zover;
dstar      = zover;
zdyad      = zover;
pzdyad     = zover;
totcors    = zover;

for ii = 1:size(overall,1);
for jj = 1:size(overall,2);
if (ii ~= jj);

% significance levels of the overall correlations
nstarovr(ii,jj) = 2 * n / (1+iccross(ii,ii)*iccross(jj,jj)+iccross(ii,jj).^2);
zover(ii,jj) = overall(ii,jj)*sqrt(nstarovr(ii,jj));
pzover(ii,jj) = (1 - normp(abs(zover(ii,jj)))) * 2;

% significance levels of the cross-intraclass correlations
nstarcrs(ii,jj) = 2*n / (1+iccross(ii,ii)*iccross(jj,jj)+overall(ii,jj).^2);
zcrs(ii,jj) = iccross(ii,jj)*sqrt(nstarcrs(ii,jj));
pzcrs(ii,jj) = (1 - normp(abs(zcrs(ii,jj)))) * 2;

% individual-level correlations
indcorrs(ii,jj)=(overall(ii,jj)-iccross(ii,jj))/ ...
                (sqrt(1-iccross(ii,ii))*sqrt(1-iccross(jj,jj)));
if (abs(indcorrs(ii,jj)) < 1);
tind(ii,jj) = (indcorrs(ii,jj)*sqrt(n-1)) / sqrt(1 - (indcorrs(ii,jj).^2));
ptind(ii,jj) = (1 - tp(abs(tind(ii,jj)),(n-1))) * 2;
end;

% dyad-level correlations
if (iccross(ii,ii) > 0 & iccross(jj,jj) > 0);
dyadcors(ii,jj)=iccross(ii,jj)/(sqrt(iccross(ii,ii))*sqrt(iccross(jj,jj)));
dstar(ii,jj)  = (2*n / (1+iccross(ii,ii)*iccross(jj,jj)+overall(ii,jj).^2)) ...
                        *iccross(ii,ii)*iccross(jj,jj);
zdyad(ii,jj)  = dyadcors(ii,jj)*sqrt(dstar(ii,jj));
pzdyad(ii,jj) = (1 - normp(abs(zdyad(ii,jj)))) * 2;
end;

% total or mean-level correlations
 totcors(ii,jj) = -9999;
if (iccross(ii,ii) ~= -1 & iccross(jj,jj) ~= -1);
totcors(ii,jj)=(overall(ii,jj)*iccross(ii,jj))/ ...
   (sqrt(1+iccross(ii,ii))*sqrt(1+iccross(jj,jj)));
end;

end;
end;
end;

% preparing the output matrices
overzp   = repmat(-9999,1,5);
iccroszp = overzp;
indrzp   = repmat(-9999,1,6);
dyadrzp  = overzp;
totr     = repmat(-9999,1,3);

for cc = 1:size(overall,2)-1;
for rr = (cc+1):size(overall,1);
overzp=[ overzp ; cc, rr, overall(cc,rr), zover(cc,rr), pzover(cc,rr) ];
iccroszp=[ iccroszp ; cc, rr, iccross(cc,rr), zcrs(cc,rr), pzcrs(cc,rr) ];
indrzp=[ indrzp ; cc, rr, indcorrs(cc,rr), tind(cc,rr), (n-1), ptind(cc,rr) ];
dyadrzp=[ dyadrzp ; cc, rr, dyadcors(cc,rr), zdyad(cc,rr), pzdyad(cc,rr) ];
totr=[ totr ; cc, rr, totcors(cc,rr) ];
end;
end;
overzp   = overzp(2:size(overzp,1),:);
iccroszp = iccroszp(2:size(iccroszp,1),:);
indrzp   = indrzp(2:size(indrzp,1),:);
dyadrzp  = dyadrzp(2:size(dyadrzp,1),:);
totr     = totr(2:size(totr,1),:);

disp([' ']);disp([' ']);
disp(['Correlational Analyses of Exchangeable-Case Dyadic Data']);disp([' ']);
disp(['Griffin & Gonzalez, 1995, Psychological Bulletin, 118, 430-439.']);disp([' ']);
disp(['All Significance Tests Are Two-Tailed']);disp([' ']);
disp(['Number of Individual Cases in the Data File = ' num2str(norig) ]);disp([' ']);
disp(['Number of Individual Cases That Will be Processed = ' num2str(size(data,1)) ]);disp([' ']);
disp([' ']);disp(['Intraclass Correlations, z-values & p-levels:']);disp([' ']);
disp(['     Var.#       r         z         p']);
disp([ (1:size(intra,1))' intra zintra pzintra] );disp([' ']);disp([' ']);
disp(['Overall Correlations, z-values & p-Levels:']);disp([' ']);
disp(['     Var.#     Var.#       r        z         p']);
disp([overzp ]);disp([' ']);disp([' ']);
disp(['Cross-Intraclass Correlations, z-values & p-Levels:']);disp([' ']);
disp(['     Var.#     Var.#       r        z         p']);
disp([iccroszp ]);disp([' ']);disp([' ']);
disp(['Individual-Level Correlations, z-values & p-Levels:']);disp([' ']);
disp(['     Var.#     Var.#       r        t        df         p']);
disp([indrzp ]);disp([' ']);disp([' ']);
disp(['Dyad-Level Correlations, z-values & p-Levels:']);disp([' ']);
disp(['     Var.#     Var.#       r        z         p']);
disp([dyadrzp ]);disp([' ']);disp([' ']);
disp(['Total or Mean-Level Correlations:' ]);disp([' ']);
disp(['     Var.#     Var.#       r']);
disp([totr ]);disp([' ']);disp([' ']);
disp(['-9999 indicates that a value could not be computed.' ]);disp([' ']);
disp(['The computations sometimes produce dyad-level' ]);
disp(['correlations > 1.00 or < -1.00.  This is most likely to' ]);
disp(['occur when an intraclass correlations is weak.' ]); 
disp(['See Griffin & Gonzalez, 1995, p. 434-437 for details.']);


